package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.DeleteEnum;
import com.zanxiang.common.enums.StatusEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.mybatis.entity.GamePayWay;
import com.zanxiang.mybatis.mapper.GamePayWayMapper;
import com.zanxiang.sdk.domain.dto.GamePayWayDTO;
import com.zanxiang.sdk.service.GamePayWayService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @author xufeng
 * @date 2022/6/13 11:46
 */
@Slf4j
@Service
public class GamePayWayServiceImpl extends ServiceImpl<GamePayWayMapper, GamePayWay> implements GamePayWayService {

    @Override
    public GamePayWay getInfo(String id) {
        return getOne(new LambdaQueryWrapper<GamePayWay>().eq(GamePayWay::getId, id).eq(GamePayWay::getIsDelete, DeleteEnum.NO.getCode()));
    }

    @Override
    public GamePayWay getPayWayToOrderPay(String gameId, Integer payType) {
        GamePayWay payInfo = getOne(new LambdaQueryWrapper<GamePayWay>()
                .eq(GamePayWay::getGameId, gameId)
                .eq(GamePayWay::getPayWayId, payType)
                .eq(GamePayWay::getIsDelete, DeleteEnum.NO.getCode())
                .eq(GamePayWay::getStatus, StatusEnum.YES.getCode())
                .last("limit 1")
                .orderByDesc(GamePayWay::getCreateTime));
        if (payInfo == null) {
            throw new RuntimeException("暂无该游戏支付方式 gameId:" + gameId + " payType:" + payType);
        }
        if (StringUtils.isEmpty(payInfo.getPayConfig())) {
            throw new RuntimeException("游戏支付方式为空 gameId:" + gameId + " payType:" + payType);
        }
        return payInfo;
    }

    @Override
    public GamePayWayDTO getGamePayWay(String gameId, Long payWayId, Long payDeviceId) {
        GamePayWay gamePayWay = getOne(new LambdaQueryWrapper<GamePayWay>()
                .eq(GamePayWay::getGameId, gameId)
                .eq(GamePayWay::getPayWayId, payWayId)
                .eq(GamePayWay::getPayDeviceId, payDeviceId)
                .eq(GamePayWay::getIsDelete, DeleteEnum.NO.getCode())
                .eq(GamePayWay::getStatus, StatusEnum.YES.getCode()));
        if (gamePayWay == null) {
            log.error("参数错误, 游戏支付配置信息不存在, gameId : {}, payWayId : {}, payDeviceId : {}", gameId, payWayId, payDeviceId);
            throw new BaseException("参数错误, 游戏支付配置信息不存在");
        }
        return BeanUtil.copy(gamePayWay, GamePayWayDTO.class);
    }
}
