package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.enums.PayDeviceEnum;
import com.zanxiang.common.enums.PayWayEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.SpringUtils;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.web.util.IpUtil;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.mybatis.entity.OrderPayParam;
import com.zanxiang.mybatis.entity.PayMerchant;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.params.ProductPayParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.vo.PayParamVO;
import com.zanxiang.sdk.enums.PayTypeEnum;
import com.zanxiang.sdk.service.*;
import com.zanxiang.sdk.service.pay.PayBaseService;
import com.zanxiang.sdk.util.WxPayUtil;
import com.zanxiang.sdk.util.XmlUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-09-28
 * @description : 订单支付
 */
@Slf4j
@Service
public class OrderPayServiceImpl implements OrderPayService {

    @Autowired
    public OrderService orderService;

    @Autowired
    public GamePayWayService gamePayWayService;

    @Autowired
    private OrderPayParamService orderPayParamService;

    @Autowired
    private PayMerchantService payMerchantService;

    @Override
    public PayParamVO getPayParam(String code, String orderId, HttpServletRequest request) {
        Order order = orderService.getOne(new LambdaQueryWrapper<Order>()
                .eq(Order::getOrderId, orderId));
        if (order == null) {
            throw new BaseException("参数错误, 订单信息不存在");
        }
        OrderPayParam orderPayParam = orderPayParamService.getOne(new LambdaQueryWrapper<OrderPayParam>()
                .eq(OrderPayParam::getOrderId, order.getOrderId()));
        if (orderPayParam != null) {
            return PayParamVO.builder()
                    .orderId(orderPayParam.getOrderId())
                    .appId(orderPayParam.getAppId())
                    .payParam(orderPayParam.getPayParam())
                    .build();
        }
        //创建支付参数
        ProductPayParamBO bo = ProductPayParamBO.builder()
                .gameId(order.getGameId().toString())
                .userId(order.getUserId())
                .payDevice(order.getPayDevice())
                .deviceSystem(order.getDeviceSystem())
                .spbillCreateIp(IpUtil.getRealIp(request))
                .outTradeNo(order.getOrderId())
                .payWay(PayWayEnum.WX_PAY.getPayWayId())
                .code(code)
                .build();
        //调起支付
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(bo.getPayWay());
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        service.payCreate(bo);
        //重新查询
        orderPayParam = orderPayParamService.getOne(new LambdaQueryWrapper<OrderPayParam>()
                .eq(OrderPayParam::getOrderId, order.getOrderId()));
        if (orderPayParam == null) {
            throw new BaseException("支付参数获取失败");
        }
        return PayParamVO.builder()
                .orderId(orderPayParam.getOrderId())
                .appId(orderPayParam.getAppId())
                .payParam(orderPayParam.getPayParam())
                .build();
    }

    @Override
    public Integer getAppletPayDevice(UserData userData) {
        return 6;
    }

    @Override
    public ResultMap payCreate(ProductPayParam product, UserData userData) {
        //创建订单
        orderService.createOrder(product, userData);
        //判断游戏类型
        if (Objects.equals(PayDeviceEnum.APPLET_PAY.getPayDeviceId(), product.getPayDevice())
                || Objects.equals(PayDeviceEnum.CUSTOM_PAY.getPayDeviceId(), product.getPayDevice())) {
            log.error("单独生成订单, 不下单");
            return ResultMap.ok(product.getOrderId());
        }
        //创建支付参数
        ProductPayParamBO bo = new ProductPayParamBO();
        bo.setGameId(userData.getGameId().toString());
        bo.setUserId(userData.getUserId());
        bo.setPayDevice(product.getPayDevice() == null ? userData.getDeviceType() : product.getPayDevice());
        bo.setDeviceSystem(userData.getDeviceSystem());
        bo.setSpbillCreateIp(userData.getIp());
        bo.setOutTradeNo(product.getOrderId());
        bo.setPayWay(product.getPayWay());
        bo.setCode(product.getCode());
        //调起支付
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(bo.getPayWay());
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        return service.payCreate(bo);
    }

    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response, Integer payType) throws IOException {
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(payType);
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        return service.notify(request, response);
    }

    @Override
    public String appletNotify(HttpServletRequest request, HttpServletResponse response) throws IOException {
        // 读取参数
        InputStream inputStream = request.getInputStream();
        String requestStr = IOUtils.toString(inputStream, StandardCharsets.UTF_8);
        // 解析xml成map
        Map<String, String> packageParams = XmlUtil.xmlToMap(requestStr);
        //获取订单信息
        String attachStr = packageParams.get("attach");
        Map<String, String> attachStrMap = JsonUtil.toMap(attachStr, Map.class, String.class);
        if (attachStrMap == null || attachStrMap.isEmpty()) {
            log.info("回调参数中attach值为空");
            return null;
        }
        //商户号id
        String merchantNo = attachStrMap.get("merchantNo");
        PayMerchant payMerchant = payMerchantService.getOne(new LambdaQueryWrapper<PayMerchant>()
                .eq(PayMerchant::getMerchantNo, merchantNo));
        Map<String, String> payConfigMap = JsonUtil.toMap(payMerchant.getPayConfig(), Map.class, String.class);
        // 判断签名是否正确
        String mySign = WxPayUtil.generateSignature(packageParams, payConfigMap.get("apiKey"));
        if (!Objects.equals(mySign, packageParams.get("sign"))) {
            log.error("通知签名验证失败, muSign : {}, sign : {}", mySign, packageParams.get("sign"));
            return null;
        }
        log.info("微信支付成功回调");
        // 处理业务开始
        Map<String, String> xmlMap = new HashMap<>();
        xmlMap.put("return_code", "SUCCESS");
        xmlMap.put("return_msg", "OK");
        // 处理业务完毕
        BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());
        log.info("返回给腾讯的回调结果, ss : {}", XmlUtil.mapToXml(xmlMap));
        out.write(XmlUtil.mapToXml(xmlMap).getBytes());
        out.flush();
        out.close();
        return HttpStatusEnum.SUCCESS.getMsg();
    }

    @Override
    public ResultMap synNotify(HttpServletRequest request, Integer payType) {
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(payType);
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        return service.synNotify(request);
    }

    @Override
    public Boolean payResult(String orderId) {
        Order order = orderService.getOne(new LambdaQueryWrapper<Order>()
                .eq(Order::getOrderId, orderId));
        if (order == null) {
            throw new BaseException("参数错误, 订单信息不存在");
        }
        if (Objects.equals(order.getStatus(), OrderStateEnum.SUCCESS.getCode())) {
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }
}
